# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

# Analytics Game Telemetry (1.11.0)

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from .....core import Operation
from .....core import HeaderStr
from .....core import HttpResponse

from ...models import HTTPValidationError


class ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut(
    Operation
):
    """Protected Update Playtime (protected_update_playtime_game_telemetry_v1_protected_steamIds__steamId__playtime__playtime__put)

    This endpoint requires valid JWT token.
    This endpoint does not require permission.

    This endpoint update player's total playtime in a specific game (AppId) from service's cache.

    Properties:
        url: /game-telemetry/v1/protected/steamIds/{steamId}/playtime/{playtime}

        method: PUT

        tags: ["Gametelemetry Operations"]

        consumes: []

        produces: ["application/json"]

        securities: [COOKIE_AUTH] or [BEARER_AUTH]

        cookie: (Cookie) OPTIONAL Union[str, HeaderStr] in header

        playtime: (playtime) REQUIRED str in path

        steam_id: (steamId) REQUIRED str in path

    Responses:
        200: OK - (Successful Response)

        422: Unprocessable Entity - HTTPValidationError (Validation Error)
    """

    # region fields

    _url: str = "/game-telemetry/v1/protected/steamIds/{steamId}/playtime/{playtime}"
    _method: str = "PUT"
    _consumes: List[str] = []
    _produces: List[str] = ["application/json"]
    _securities: List[List[str]] = [["COOKIE_AUTH"], ["BEARER_AUTH"]]
    _location_query: str = None

    cookie: Union[str, HeaderStr]  # OPTIONAL in [header]
    playtime: str  # REQUIRED in [path]
    steam_id: str  # REQUIRED in [path]

    # endregion fields

    # region properties

    @property
    def url(self) -> str:
        return self._url

    @property
    def method(self) -> str:
        return self._method

    @property
    def consumes(self) -> List[str]:
        return self._consumes

    @property
    def produces(self) -> List[str]:
        return self._produces

    @property
    def securities(self) -> List[List[str]]:
        return self._securities

    @property
    def location_query(self) -> str:
        return self._location_query

    # endregion properties

    # region get methods

    # endregion get methods

    # region get_x_params methods

    def get_all_params(self) -> dict:
        return {
            "header": self.get_header_params(),
            "path": self.get_path_params(),
        }

    def get_header_params(self) -> dict:
        result = {}
        if hasattr(self, "cookie"):
            result["Cookie"] = self.cookie
        return result

    def get_path_params(self) -> dict:
        result = {}
        if hasattr(self, "playtime"):
            result["playtime"] = self.playtime
        if hasattr(self, "steam_id"):
            result["steamId"] = self.steam_id
        return result

    # endregion get_x_params methods

    # region is/has methods

    # endregion is/has methods

    # region with_x methods

    def with_cookie(
        self, value: Union[str, HeaderStr]
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.cookie = value
        return self

    def with_cookie_access_token(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        if not hasattr(self, "cookie"):
            self.cookie = HeaderStr()
        self.cookie["access_token"] = value
        return self

    def with_playtime(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.playtime = value
        return self

    def with_steam_id(
        self, value: str
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        self.steam_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "cookie") and self.cookie:
            result["Cookie"] = str(self.cookie)
        elif include_empty:
            result["Cookie"] = ""
        if hasattr(self, "playtime") and self.playtime:
            result["playtime"] = str(self.playtime)
        elif include_empty:
            result["playtime"] = ""
        if hasattr(self, "steam_id") and self.steam_id:
            result["steamId"] = str(self.steam_id)
        elif include_empty:
            result["steamId"] = ""
        return result

    # endregion to methods

    # region response methods

    # noinspection PyMethodMayBeStatic
    def parse_response(
        self, code: int, content_type: str, content: Any
    ) -> Tuple[
        Union[None, HttpResponse], Union[None, HTTPValidationError, HttpResponse]
    ]:
        """Parse the given response.

        200: OK - (Successful Response)

        422: Unprocessable Entity - HTTPValidationError (Validation Error)

        ---: HttpResponse (Undocumented Response)

        ---: HttpResponse (Unexpected Content-Type Error)

        ---: HttpResponse (Unhandled Error)
        """
        pre_processed_response, error = self.pre_process_response(
            code=code, content_type=content_type, content=content
        )
        if error is not None:
            return None, None if error.is_no_content() else error
        code, content_type, content = pre_processed_response

        if code == 200:
            return HttpResponse.create(code, "OK"), None
        if code == 422:
            return None, HTTPValidationError.create_from_dict(content)

        return self.handle_undocumented_response(
            code=code, content_type=content_type, content=content
        )

    # endregion response methods

    # region static methods

    @classmethod
    def create(
        cls,
        playtime: str,
        steam_id: str,
        cookie: Optional[Union[str, HeaderStr]] = None,
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        instance = cls()
        instance.playtime = playtime
        instance.steam_id = steam_id
        if cookie is not None:
            instance.cookie = cookie
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ProtectedUpdatePlaytimeGameTelemetryV1ProtectedSteamIdsSteamIdPlaytimePlaytimePut:
        instance = cls()
        if "Cookie" in dict_ and dict_["Cookie"] is not None:
            instance.cookie = str(dict_["Cookie"])
        elif include_empty:
            instance.cookie = ""
        if "playtime" in dict_ and dict_["playtime"] is not None:
            instance.playtime = str(dict_["playtime"])
        elif include_empty:
            instance.playtime = ""
        if "steamId" in dict_ and dict_["steamId"] is not None:
            instance.steam_id = str(dict_["steamId"])
        elif include_empty:
            instance.steam_id = ""
        return instance

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "Cookie": "cookie",
            "playtime": "playtime",
            "steamId": "steam_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "Cookie": False,
            "playtime": True,
            "steamId": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.17.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_result_attribute_response import ModelsResultAttributeResponse


class ModelsPlayerResultResponse(Model):
    """Models player result response (models.PlayerResultResponse)

    Properties:
        current_score: (current_score) REQUIRED List[ModelsResultAttributeResponse]

        user_id: (user_id) REQUIRED str
    """

    # region fields

    current_score: List[ModelsResultAttributeResponse]  # REQUIRED
    user_id: str  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_current_score(
        self, value: List[ModelsResultAttributeResponse]
    ) -> ModelsPlayerResultResponse:
        self.current_score = value
        return self

    def with_user_id(self, value: str) -> ModelsPlayerResultResponse:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "current_score"):
            result["current_score"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.current_score
            ]
        elif include_empty:
            result["current_score"] = []
        if hasattr(self, "user_id"):
            result["user_id"] = str(self.user_id)
        elif include_empty:
            result["user_id"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        current_score: List[ModelsResultAttributeResponse],
        user_id: str,
    ) -> ModelsPlayerResultResponse:
        instance = cls()
        instance.current_score = current_score
        instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsPlayerResultResponse:
        instance = cls()
        if not dict_:
            return instance
        if "current_score" in dict_ and dict_["current_score"] is not None:
            instance.current_score = [
                ModelsResultAttributeResponse.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["current_score"]
            ]
        elif include_empty:
            instance.current_score = []
        if "user_id" in dict_ and dict_["user_id"] is not None:
            instance.user_id = str(dict_["user_id"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsPlayerResultResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsPlayerResultResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsPlayerResultResponse,
        List[ModelsPlayerResultResponse],
        Dict[Any, ModelsPlayerResultResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "current_score": "current_score",
            "user_id": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "current_score": True,
            "user_id": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.17.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.billing_account import BillingAccount
from ..models.currency_summary import CurrencySummary
from ..models.entitlement_summary import EntitlementSummary
from ..models.item_snapshot import ItemSnapshot
from ..models.recurring import Recurring


class ChargeStatusEnum(StrEnum):
    CHARGED = "CHARGED"
    CHARGE_FAILED = "CHARGE_FAILED"
    NEVER = "NEVER"
    RECURRING_CHARGING = "RECURRING_CHARGING"
    SETUP = "SETUP"


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    CANCELLED = "CANCELLED"
    EXPIRED = "EXPIRED"
    INIT = "INIT"


class SubscribedByEnum(StrEnum):
    PLATFORM = "PLATFORM"
    USER = "USER"


class SubscriptionInfo(Model):
    """Subscription info (SubscriptionInfo)

    Properties:
        charge_status: (chargeStatus) REQUIRED Union[str, ChargeStatusEnum]

        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        in_fixed_cycle_trial: (inFixedCycleTrial) REQUIRED bool

        in_fixed_free_days: (inFixedFreeDays) REQUIRED bool

        item_id: (itemId) REQUIRED str

        namespace: (namespace) REQUIRED str

        payment_flow_required: (paymentFlowRequired) REQUIRED bool

        recurring: (recurring) REQUIRED Recurring

        status: (status) REQUIRED Union[str, StatusEnum]

        updated_at: (updatedAt) REQUIRED str

        user_id: (userId) REQUIRED str

        billing_account: (billingAccount) OPTIONAL BillingAccount

        charged_cycles: (chargedCycles) OPTIONAL int

        currency: (currency) OPTIONAL CurrencySummary

        current_cycle: (currentCycle) OPTIONAL int

        current_period_end: (currentPeriodEnd) OPTIONAL str

        current_period_start: (currentPeriodStart) OPTIONAL str

        description: (description) OPTIONAL str

        end: (end) OPTIONAL str

        entitlements: (entitlements) OPTIONAL List[EntitlementSummary]

        first_subscribe: (firstSubscribe) OPTIONAL bool

        item_snapshot: (itemSnapshot) OPTIONAL ItemSnapshot

        language: (language) OPTIONAL str

        next_billing_date: (nextBillingDate) OPTIONAL str

        paid: (paid) OPTIONAL bool

        payment_order_no: (paymentOrderNo) OPTIONAL str

        payment_station_url: (paymentStationUrl) OPTIONAL str

        price: (price) OPTIONAL int

        region: (region) OPTIONAL str

        retry_attempted: (retryAttempted) OPTIONAL int

        return_url: (returnUrl) OPTIONAL str

        sandbox: (sandbox) OPTIONAL bool

        sku: (sku) OPTIONAL str

        source: (source) OPTIONAL str

        start: (start) OPTIONAL str

        subscribed_at: (subscribedAt) OPTIONAL str

        subscribed_by: (subscribedBy) OPTIONAL Union[str, SubscribedByEnum]

        title: (title) OPTIONAL str

        trial_price: (trialPrice) OPTIONAL int

        trialed_cycles: (trialedCycles) OPTIONAL int

        unsubscribe_reason: (unsubscribeReason) OPTIONAL str

        unsubscribed_at: (unsubscribedAt) OPTIONAL str
    """

    # region fields

    charge_status: Union[str, ChargeStatusEnum]  # REQUIRED
    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    in_fixed_cycle_trial: bool  # REQUIRED
    in_fixed_free_days: bool  # REQUIRED
    item_id: str  # REQUIRED
    namespace: str  # REQUIRED
    payment_flow_required: bool  # REQUIRED
    recurring: Recurring  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    updated_at: str  # REQUIRED
    user_id: str  # REQUIRED
    billing_account: BillingAccount  # OPTIONAL
    charged_cycles: int  # OPTIONAL
    currency: CurrencySummary  # OPTIONAL
    current_cycle: int  # OPTIONAL
    current_period_end: str  # OPTIONAL
    current_period_start: str  # OPTIONAL
    description: str  # OPTIONAL
    end: str  # OPTIONAL
    entitlements: List[EntitlementSummary]  # OPTIONAL
    first_subscribe: bool  # OPTIONAL
    item_snapshot: ItemSnapshot  # OPTIONAL
    language: str  # OPTIONAL
    next_billing_date: str  # OPTIONAL
    paid: bool  # OPTIONAL
    payment_order_no: str  # OPTIONAL
    payment_station_url: str  # OPTIONAL
    price: int  # OPTIONAL
    region: str  # OPTIONAL
    retry_attempted: int  # OPTIONAL
    return_url: str  # OPTIONAL
    sandbox: bool  # OPTIONAL
    sku: str  # OPTIONAL
    source: str  # OPTIONAL
    start: str  # OPTIONAL
    subscribed_at: str  # OPTIONAL
    subscribed_by: Union[str, SubscribedByEnum]  # OPTIONAL
    title: str  # OPTIONAL
    trial_price: int  # OPTIONAL
    trialed_cycles: int  # OPTIONAL
    unsubscribe_reason: str  # OPTIONAL
    unsubscribed_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_charge_status(
        self, value: Union[str, ChargeStatusEnum]
    ) -> SubscriptionInfo:
        self.charge_status = value
        return self

    def with_created_at(self, value: str) -> SubscriptionInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> SubscriptionInfo:
        self.id_ = value
        return self

    def with_in_fixed_cycle_trial(self, value: bool) -> SubscriptionInfo:
        self.in_fixed_cycle_trial = value
        return self

    def with_in_fixed_free_days(self, value: bool) -> SubscriptionInfo:
        self.in_fixed_free_days = value
        return self

    def with_item_id(self, value: str) -> SubscriptionInfo:
        self.item_id = value
        return self

    def with_namespace(self, value: str) -> SubscriptionInfo:
        self.namespace = value
        return self

    def with_payment_flow_required(self, value: bool) -> SubscriptionInfo:
        self.payment_flow_required = value
        return self

    def with_recurring(self, value: Recurring) -> SubscriptionInfo:
        self.recurring = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> SubscriptionInfo:
        self.status = value
        return self

    def with_updated_at(self, value: str) -> SubscriptionInfo:
        self.updated_at = value
        return self

    def with_user_id(self, value: str) -> SubscriptionInfo:
        self.user_id = value
        return self

    def with_billing_account(self, value: BillingAccount) -> SubscriptionInfo:
        self.billing_account = value
        return self

    def with_charged_cycles(self, value: int) -> SubscriptionInfo:
        self.charged_cycles = value
        return self

    def with_currency(self, value: CurrencySummary) -> SubscriptionInfo:
        self.currency = value
        return self

    def with_current_cycle(self, value: int) -> SubscriptionInfo:
        self.current_cycle = value
        return self

    def with_current_period_end(self, value: str) -> SubscriptionInfo:
        self.current_period_end = value
        return self

    def with_current_period_start(self, value: str) -> SubscriptionInfo:
        self.current_period_start = value
        return self

    def with_description(self, value: str) -> SubscriptionInfo:
        self.description = value
        return self

    def with_end(self, value: str) -> SubscriptionInfo:
        self.end = value
        return self

    def with_entitlements(self, value: List[EntitlementSummary]) -> SubscriptionInfo:
        self.entitlements = value
        return self

    def with_first_subscribe(self, value: bool) -> SubscriptionInfo:
        self.first_subscribe = value
        return self

    def with_item_snapshot(self, value: ItemSnapshot) -> SubscriptionInfo:
        self.item_snapshot = value
        return self

    def with_language(self, value: str) -> SubscriptionInfo:
        self.language = value
        return self

    def with_next_billing_date(self, value: str) -> SubscriptionInfo:
        self.next_billing_date = value
        return self

    def with_paid(self, value: bool) -> SubscriptionInfo:
        self.paid = value
        return self

    def with_payment_order_no(self, value: str) -> SubscriptionInfo:
        self.payment_order_no = value
        return self

    def with_payment_station_url(self, value: str) -> SubscriptionInfo:
        self.payment_station_url = value
        return self

    def with_price(self, value: int) -> SubscriptionInfo:
        self.price = value
        return self

    def with_region(self, value: str) -> SubscriptionInfo:
        self.region = value
        return self

    def with_retry_attempted(self, value: int) -> SubscriptionInfo:
        self.retry_attempted = value
        return self

    def with_return_url(self, value: str) -> SubscriptionInfo:
        self.return_url = value
        return self

    def with_sandbox(self, value: bool) -> SubscriptionInfo:
        self.sandbox = value
        return self

    def with_sku(self, value: str) -> SubscriptionInfo:
        self.sku = value
        return self

    def with_source(self, value: str) -> SubscriptionInfo:
        self.source = value
        return self

    def with_start(self, value: str) -> SubscriptionInfo:
        self.start = value
        return self

    def with_subscribed_at(self, value: str) -> SubscriptionInfo:
        self.subscribed_at = value
        return self

    def with_subscribed_by(
        self, value: Union[str, SubscribedByEnum]
    ) -> SubscriptionInfo:
        self.subscribed_by = value
        return self

    def with_title(self, value: str) -> SubscriptionInfo:
        self.title = value
        return self

    def with_trial_price(self, value: int) -> SubscriptionInfo:
        self.trial_price = value
        return self

    def with_trialed_cycles(self, value: int) -> SubscriptionInfo:
        self.trialed_cycles = value
        return self

    def with_unsubscribe_reason(self, value: str) -> SubscriptionInfo:
        self.unsubscribe_reason = value
        return self

    def with_unsubscribed_at(self, value: str) -> SubscriptionInfo:
        self.unsubscribed_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "charge_status"):
            result["chargeStatus"] = str(self.charge_status)
        elif include_empty:
            result["chargeStatus"] = Union[str, ChargeStatusEnum]()
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "in_fixed_cycle_trial"):
            result["inFixedCycleTrial"] = bool(self.in_fixed_cycle_trial)
        elif include_empty:
            result["inFixedCycleTrial"] = False
        if hasattr(self, "in_fixed_free_days"):
            result["inFixedFreeDays"] = bool(self.in_fixed_free_days)
        elif include_empty:
            result["inFixedFreeDays"] = False
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "payment_flow_required"):
            result["paymentFlowRequired"] = bool(self.payment_flow_required)
        elif include_empty:
            result["paymentFlowRequired"] = False
        if hasattr(self, "recurring"):
            result["recurring"] = self.recurring.to_dict(include_empty=include_empty)
        elif include_empty:
            result["recurring"] = Recurring()
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "billing_account"):
            result["billingAccount"] = self.billing_account.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["billingAccount"] = BillingAccount()
        if hasattr(self, "charged_cycles"):
            result["chargedCycles"] = int(self.charged_cycles)
        elif include_empty:
            result["chargedCycles"] = 0
        if hasattr(self, "currency"):
            result["currency"] = self.currency.to_dict(include_empty=include_empty)
        elif include_empty:
            result["currency"] = CurrencySummary()
        if hasattr(self, "current_cycle"):
            result["currentCycle"] = int(self.current_cycle)
        elif include_empty:
            result["currentCycle"] = 0
        if hasattr(self, "current_period_end"):
            result["currentPeriodEnd"] = str(self.current_period_end)
        elif include_empty:
            result["currentPeriodEnd"] = ""
        if hasattr(self, "current_period_start"):
            result["currentPeriodStart"] = str(self.current_period_start)
        elif include_empty:
            result["currentPeriodStart"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "end"):
            result["end"] = str(self.end)
        elif include_empty:
            result["end"] = ""
        if hasattr(self, "entitlements"):
            result["entitlements"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.entitlements
            ]
        elif include_empty:
            result["entitlements"] = []
        if hasattr(self, "first_subscribe"):
            result["firstSubscribe"] = bool(self.first_subscribe)
        elif include_empty:
            result["firstSubscribe"] = False
        if hasattr(self, "item_snapshot"):
            result["itemSnapshot"] = self.item_snapshot.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["itemSnapshot"] = ItemSnapshot()
        if hasattr(self, "language"):
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        if hasattr(self, "next_billing_date"):
            result["nextBillingDate"] = str(self.next_billing_date)
        elif include_empty:
            result["nextBillingDate"] = ""
        if hasattr(self, "paid"):
            result["paid"] = bool(self.paid)
        elif include_empty:
            result["paid"] = False
        if hasattr(self, "payment_order_no"):
            result["paymentOrderNo"] = str(self.payment_order_no)
        elif include_empty:
            result["paymentOrderNo"] = ""
        if hasattr(self, "payment_station_url"):
            result["paymentStationUrl"] = str(self.payment_station_url)
        elif include_empty:
            result["paymentStationUrl"] = ""
        if hasattr(self, "price"):
            result["price"] = int(self.price)
        elif include_empty:
            result["price"] = 0
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "retry_attempted"):
            result["retryAttempted"] = int(self.retry_attempted)
        elif include_empty:
            result["retryAttempted"] = 0
        if hasattr(self, "return_url"):
            result["returnUrl"] = str(self.return_url)
        elif include_empty:
            result["returnUrl"] = ""
        if hasattr(self, "sandbox"):
            result["sandbox"] = bool(self.sandbox)
        elif include_empty:
            result["sandbox"] = False
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "start"):
            result["start"] = str(self.start)
        elif include_empty:
            result["start"] = ""
        if hasattr(self, "subscribed_at"):
            result["subscribedAt"] = str(self.subscribed_at)
        elif include_empty:
            result["subscribedAt"] = ""
        if hasattr(self, "subscribed_by"):
            result["subscribedBy"] = str(self.subscribed_by)
        elif include_empty:
            result["subscribedBy"] = Union[str, SubscribedByEnum]()
        if hasattr(self, "title"):
            result["title"] = str(self.title)
        elif include_empty:
            result["title"] = ""
        if hasattr(self, "trial_price"):
            result["trialPrice"] = int(self.trial_price)
        elif include_empty:
            result["trialPrice"] = 0
        if hasattr(self, "trialed_cycles"):
            result["trialedCycles"] = int(self.trialed_cycles)
        elif include_empty:
            result["trialedCycles"] = 0
        if hasattr(self, "unsubscribe_reason"):
            result["unsubscribeReason"] = str(self.unsubscribe_reason)
        elif include_empty:
            result["unsubscribeReason"] = ""
        if hasattr(self, "unsubscribed_at"):
            result["unsubscribedAt"] = str(self.unsubscribed_at)
        elif include_empty:
            result["unsubscribedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        charge_status: Union[str, ChargeStatusEnum],
        created_at: str,
        id_: str,
        in_fixed_cycle_trial: bool,
        in_fixed_free_days: bool,
        item_id: str,
        namespace: str,
        payment_flow_required: bool,
        recurring: Recurring,
        status: Union[str, StatusEnum],
        updated_at: str,
        user_id: str,
        billing_account: Optional[BillingAccount] = None,
        charged_cycles: Optional[int] = None,
        currency: Optional[CurrencySummary] = None,
        current_cycle: Optional[int] = None,
        current_period_end: Optional[str] = None,
        current_period_start: Optional[str] = None,
        description: Optional[str] = None,
        end: Optional[str] = None,
        entitlements: Optional[List[EntitlementSummary]] = None,
        first_subscribe: Optional[bool] = None,
        item_snapshot: Optional[ItemSnapshot] = None,
        language: Optional[str] = None,
        next_billing_date: Optional[str] = None,
        paid: Optional[bool] = None,
        payment_order_no: Optional[str] = None,
        payment_station_url: Optional[str] = None,
        price: Optional[int] = None,
        region: Optional[str] = None,
        retry_attempted: Optional[int] = None,
        return_url: Optional[str] = None,
        sandbox: Optional[bool] = None,
        sku: Optional[str] = None,
        source: Optional[str] = None,
        start: Optional[str] = None,
        subscribed_at: Optional[str] = None,
        subscribed_by: Optional[Union[str, SubscribedByEnum]] = None,
        title: Optional[str] = None,
        trial_price: Optional[int] = None,
        trialed_cycles: Optional[int] = None,
        unsubscribe_reason: Optional[str] = None,
        unsubscribed_at: Optional[str] = None,
    ) -> SubscriptionInfo:
        instance = cls()
        instance.charge_status = charge_status
        instance.created_at = created_at
        instance.id_ = id_
        instance.in_fixed_cycle_trial = in_fixed_cycle_trial
        instance.in_fixed_free_days = in_fixed_free_days
        instance.item_id = item_id
        instance.namespace = namespace
        instance.payment_flow_required = payment_flow_required
        instance.recurring = recurring
        instance.status = status
        instance.updated_at = updated_at
        instance.user_id = user_id
        if billing_account is not None:
            instance.billing_account = billing_account
        if charged_cycles is not None:
            instance.charged_cycles = charged_cycles
        if currency is not None:
            instance.currency = currency
        if current_cycle is not None:
            instance.current_cycle = current_cycle
        if current_period_end is not None:
            instance.current_period_end = current_period_end
        if current_period_start is not None:
            instance.current_period_start = current_period_start
        if description is not None:
            instance.description = description
        if end is not None:
            instance.end = end
        if entitlements is not None:
            instance.entitlements = entitlements
        if first_subscribe is not None:
            instance.first_subscribe = first_subscribe
        if item_snapshot is not None:
            instance.item_snapshot = item_snapshot
        if language is not None:
            instance.language = language
        if next_billing_date is not None:
            instance.next_billing_date = next_billing_date
        if paid is not None:
            instance.paid = paid
        if payment_order_no is not None:
            instance.payment_order_no = payment_order_no
        if payment_station_url is not None:
            instance.payment_station_url = payment_station_url
        if price is not None:
            instance.price = price
        if region is not None:
            instance.region = region
        if retry_attempted is not None:
            instance.retry_attempted = retry_attempted
        if return_url is not None:
            instance.return_url = return_url
        if sandbox is not None:
            instance.sandbox = sandbox
        if sku is not None:
            instance.sku = sku
        if source is not None:
            instance.source = source
        if start is not None:
            instance.start = start
        if subscribed_at is not None:
            instance.subscribed_at = subscribed_at
        if subscribed_by is not None:
            instance.subscribed_by = subscribed_by
        if title is not None:
            instance.title = title
        if trial_price is not None:
            instance.trial_price = trial_price
        if trialed_cycles is not None:
            instance.trialed_cycles = trialed_cycles
        if unsubscribe_reason is not None:
            instance.unsubscribe_reason = unsubscribe_reason
        if unsubscribed_at is not None:
            instance.unsubscribed_at = unsubscribed_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> SubscriptionInfo:
        instance = cls()
        if not dict_:
            return instance
        if "chargeStatus" in dict_ and dict_["chargeStatus"] is not None:
            instance.charge_status = str(dict_["chargeStatus"])
        elif include_empty:
            instance.charge_status = Union[str, ChargeStatusEnum]()
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "inFixedCycleTrial" in dict_ and dict_["inFixedCycleTrial"] is not None:
            instance.in_fixed_cycle_trial = bool(dict_["inFixedCycleTrial"])
        elif include_empty:
            instance.in_fixed_cycle_trial = False
        if "inFixedFreeDays" in dict_ and dict_["inFixedFreeDays"] is not None:
            instance.in_fixed_free_days = bool(dict_["inFixedFreeDays"])
        elif include_empty:
            instance.in_fixed_free_days = False
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "paymentFlowRequired" in dict_ and dict_["paymentFlowRequired"] is not None:
            instance.payment_flow_required = bool(dict_["paymentFlowRequired"])
        elif include_empty:
            instance.payment_flow_required = False
        if "recurring" in dict_ and dict_["recurring"] is not None:
            instance.recurring = Recurring.create_from_dict(
                dict_["recurring"], include_empty=include_empty
            )
        elif include_empty:
            instance.recurring = Recurring()
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "billingAccount" in dict_ and dict_["billingAccount"] is not None:
            instance.billing_account = BillingAccount.create_from_dict(
                dict_["billingAccount"], include_empty=include_empty
            )
        elif include_empty:
            instance.billing_account = BillingAccount()
        if "chargedCycles" in dict_ and dict_["chargedCycles"] is not None:
            instance.charged_cycles = int(dict_["chargedCycles"])
        elif include_empty:
            instance.charged_cycles = 0
        if "currency" in dict_ and dict_["currency"] is not None:
            instance.currency = CurrencySummary.create_from_dict(
                dict_["currency"], include_empty=include_empty
            )
        elif include_empty:
            instance.currency = CurrencySummary()
        if "currentCycle" in dict_ and dict_["currentCycle"] is not None:
            instance.current_cycle = int(dict_["currentCycle"])
        elif include_empty:
            instance.current_cycle = 0
        if "currentPeriodEnd" in dict_ and dict_["currentPeriodEnd"] is not None:
            instance.current_period_end = str(dict_["currentPeriodEnd"])
        elif include_empty:
            instance.current_period_end = ""
        if "currentPeriodStart" in dict_ and dict_["currentPeriodStart"] is not None:
            instance.current_period_start = str(dict_["currentPeriodStart"])
        elif include_empty:
            instance.current_period_start = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "end" in dict_ and dict_["end"] is not None:
            instance.end = str(dict_["end"])
        elif include_empty:
            instance.end = ""
        if "entitlements" in dict_ and dict_["entitlements"] is not None:
            instance.entitlements = [
                EntitlementSummary.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["entitlements"]
            ]
        elif include_empty:
            instance.entitlements = []
        if "firstSubscribe" in dict_ and dict_["firstSubscribe"] is not None:
            instance.first_subscribe = bool(dict_["firstSubscribe"])
        elif include_empty:
            instance.first_subscribe = False
        if "itemSnapshot" in dict_ and dict_["itemSnapshot"] is not None:
            instance.item_snapshot = ItemSnapshot.create_from_dict(
                dict_["itemSnapshot"], include_empty=include_empty
            )
        elif include_empty:
            instance.item_snapshot = ItemSnapshot()
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        if "nextBillingDate" in dict_ and dict_["nextBillingDate"] is not None:
            instance.next_billing_date = str(dict_["nextBillingDate"])
        elif include_empty:
            instance.next_billing_date = ""
        if "paid" in dict_ and dict_["paid"] is not None:
            instance.paid = bool(dict_["paid"])
        elif include_empty:
            instance.paid = False
        if "paymentOrderNo" in dict_ and dict_["paymentOrderNo"] is not None:
            instance.payment_order_no = str(dict_["paymentOrderNo"])
        elif include_empty:
            instance.payment_order_no = ""
        if "paymentStationUrl" in dict_ and dict_["paymentStationUrl"] is not None:
            instance.payment_station_url = str(dict_["paymentStationUrl"])
        elif include_empty:
            instance.payment_station_url = ""
        if "price" in dict_ and dict_["price"] is not None:
            instance.price = int(dict_["price"])
        elif include_empty:
            instance.price = 0
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "retryAttempted" in dict_ and dict_["retryAttempted"] is not None:
            instance.retry_attempted = int(dict_["retryAttempted"])
        elif include_empty:
            instance.retry_attempted = 0
        if "returnUrl" in dict_ and dict_["returnUrl"] is not None:
            instance.return_url = str(dict_["returnUrl"])
        elif include_empty:
            instance.return_url = ""
        if "sandbox" in dict_ and dict_["sandbox"] is not None:
            instance.sandbox = bool(dict_["sandbox"])
        elif include_empty:
            instance.sandbox = False
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "start" in dict_ and dict_["start"] is not None:
            instance.start = str(dict_["start"])
        elif include_empty:
            instance.start = ""
        if "subscribedAt" in dict_ and dict_["subscribedAt"] is not None:
            instance.subscribed_at = str(dict_["subscribedAt"])
        elif include_empty:
            instance.subscribed_at = ""
        if "subscribedBy" in dict_ and dict_["subscribedBy"] is not None:
            instance.subscribed_by = str(dict_["subscribedBy"])
        elif include_empty:
            instance.subscribed_by = Union[str, SubscribedByEnum]()
        if "title" in dict_ and dict_["title"] is not None:
            instance.title = str(dict_["title"])
        elif include_empty:
            instance.title = ""
        if "trialPrice" in dict_ and dict_["trialPrice"] is not None:
            instance.trial_price = int(dict_["trialPrice"])
        elif include_empty:
            instance.trial_price = 0
        if "trialedCycles" in dict_ and dict_["trialedCycles"] is not None:
            instance.trialed_cycles = int(dict_["trialedCycles"])
        elif include_empty:
            instance.trialed_cycles = 0
        if "unsubscribeReason" in dict_ and dict_["unsubscribeReason"] is not None:
            instance.unsubscribe_reason = str(dict_["unsubscribeReason"])
        elif include_empty:
            instance.unsubscribe_reason = ""
        if "unsubscribedAt" in dict_ and dict_["unsubscribedAt"] is not None:
            instance.unsubscribed_at = str(dict_["unsubscribedAt"])
        elif include_empty:
            instance.unsubscribed_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, SubscriptionInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[SubscriptionInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[SubscriptionInfo, List[SubscriptionInfo], Dict[Any, SubscriptionInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "chargeStatus": "charge_status",
            "createdAt": "created_at",
            "id": "id_",
            "inFixedCycleTrial": "in_fixed_cycle_trial",
            "inFixedFreeDays": "in_fixed_free_days",
            "itemId": "item_id",
            "namespace": "namespace",
            "paymentFlowRequired": "payment_flow_required",
            "recurring": "recurring",
            "status": "status",
            "updatedAt": "updated_at",
            "userId": "user_id",
            "billingAccount": "billing_account",
            "chargedCycles": "charged_cycles",
            "currency": "currency",
            "currentCycle": "current_cycle",
            "currentPeriodEnd": "current_period_end",
            "currentPeriodStart": "current_period_start",
            "description": "description",
            "end": "end",
            "entitlements": "entitlements",
            "firstSubscribe": "first_subscribe",
            "itemSnapshot": "item_snapshot",
            "language": "language",
            "nextBillingDate": "next_billing_date",
            "paid": "paid",
            "paymentOrderNo": "payment_order_no",
            "paymentStationUrl": "payment_station_url",
            "price": "price",
            "region": "region",
            "retryAttempted": "retry_attempted",
            "returnUrl": "return_url",
            "sandbox": "sandbox",
            "sku": "sku",
            "source": "source",
            "start": "start",
            "subscribedAt": "subscribed_at",
            "subscribedBy": "subscribed_by",
            "title": "title",
            "trialPrice": "trial_price",
            "trialedCycles": "trialed_cycles",
            "unsubscribeReason": "unsubscribe_reason",
            "unsubscribedAt": "unsubscribed_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "chargeStatus": True,
            "createdAt": True,
            "id": True,
            "inFixedCycleTrial": True,
            "inFixedFreeDays": True,
            "itemId": True,
            "namespace": True,
            "paymentFlowRequired": True,
            "recurring": True,
            "status": True,
            "updatedAt": True,
            "userId": True,
            "billingAccount": False,
            "chargedCycles": False,
            "currency": False,
            "currentCycle": False,
            "currentPeriodEnd": False,
            "currentPeriodStart": False,
            "description": False,
            "end": False,
            "entitlements": False,
            "firstSubscribe": False,
            "itemSnapshot": False,
            "language": False,
            "nextBillingDate": False,
            "paid": False,
            "paymentOrderNo": False,
            "paymentStationUrl": False,
            "price": False,
            "region": False,
            "retryAttempted": False,
            "returnUrl": False,
            "sandbox": False,
            "sku": False,
            "source": False,
            "start": False,
            "subscribedAt": False,
            "subscribedBy": False,
            "title": False,
            "trialPrice": False,
            "trialedCycles": False,
            "unsubscribeReason": False,
            "unsubscribedAt": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "chargeStatus": [
                "CHARGED",
                "CHARGE_FAILED",
                "NEVER",
                "RECURRING_CHARGING",
                "SETUP",
            ],
            "status": ["ACTIVE", "CANCELLED", "EXPIRED", "INIT"],
            "subscribedBy": ["PLATFORM", "USER"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.17.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
import re
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class OrderCreate(Model):
    """Order create (OrderCreate)

    Properties:
        currency_code: (currencyCode) REQUIRED str

        discounted_price: (discountedPrice) REQUIRED int

        item_id: (itemId) REQUIRED str

        price: (price) REQUIRED int

        quantity: (quantity) REQUIRED int

        ext: (ext) OPTIONAL Dict[str, Any]

        language: (language) OPTIONAL str

        region: (region) OPTIONAL str

        return_url: (returnUrl) OPTIONAL str
    """

    # region fields

    currency_code: str  # REQUIRED
    discounted_price: int  # REQUIRED
    item_id: str  # REQUIRED
    price: int  # REQUIRED
    quantity: int  # REQUIRED
    ext: Dict[str, Any]  # OPTIONAL
    language: str  # OPTIONAL
    region: str  # OPTIONAL
    return_url: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_currency_code(self, value: str) -> OrderCreate:
        self.currency_code = value
        return self

    def with_discounted_price(self, value: int) -> OrderCreate:
        self.discounted_price = value
        return self

    def with_item_id(self, value: str) -> OrderCreate:
        self.item_id = value
        return self

    def with_price(self, value: int) -> OrderCreate:
        self.price = value
        return self

    def with_quantity(self, value: int) -> OrderCreate:
        self.quantity = value
        return self

    def with_ext(self, value: Dict[str, Any]) -> OrderCreate:
        self.ext = value
        return self

    def with_language(self, value: str) -> OrderCreate:
        self.language = value
        return self

    def with_region(self, value: str) -> OrderCreate:
        self.region = value
        return self

    def with_return_url(self, value: str) -> OrderCreate:
        self.return_url = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "discounted_price"):
            result["discountedPrice"] = int(self.discounted_price)
        elif include_empty:
            result["discountedPrice"] = 0
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "price"):
            result["price"] = int(self.price)
        elif include_empty:
            result["price"] = 0
        if hasattr(self, "quantity"):
            result["quantity"] = int(self.quantity)
        elif include_empty:
            result["quantity"] = 0
        if hasattr(self, "ext"):
            result["ext"] = {str(k0): v0 for k0, v0 in self.ext.items()}
        elif include_empty:
            result["ext"] = {}
        if hasattr(self, "language"):
            result["language"] = str(self.language)
        elif include_empty:
            result["language"] = ""
        if hasattr(self, "region"):
            result["region"] = str(self.region)
        elif include_empty:
            result["region"] = ""
        if hasattr(self, "return_url"):
            result["returnUrl"] = str(self.return_url)
        elif include_empty:
            result["returnUrl"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        currency_code: str,
        discounted_price: int,
        item_id: str,
        price: int,
        quantity: int,
        ext: Optional[Dict[str, Any]] = None,
        language: Optional[str] = None,
        region: Optional[str] = None,
        return_url: Optional[str] = None,
    ) -> OrderCreate:
        instance = cls()
        instance.currency_code = currency_code
        instance.discounted_price = discounted_price
        instance.item_id = item_id
        instance.price = price
        instance.quantity = quantity
        if ext is not None:
            instance.ext = ext
        if language is not None:
            instance.language = language
        if region is not None:
            instance.region = region
        if return_url is not None:
            instance.return_url = return_url
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> OrderCreate:
        instance = cls()
        if not dict_:
            return instance
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "discountedPrice" in dict_ and dict_["discountedPrice"] is not None:
            instance.discounted_price = int(dict_["discountedPrice"])
        elif include_empty:
            instance.discounted_price = 0
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "price" in dict_ and dict_["price"] is not None:
            instance.price = int(dict_["price"])
        elif include_empty:
            instance.price = 0
        if "quantity" in dict_ and dict_["quantity"] is not None:
            instance.quantity = int(dict_["quantity"])
        elif include_empty:
            instance.quantity = 0
        if "ext" in dict_ and dict_["ext"] is not None:
            instance.ext = {str(k0): v0 for k0, v0 in dict_["ext"].items()}
        elif include_empty:
            instance.ext = {}
        if "language" in dict_ and dict_["language"] is not None:
            instance.language = str(dict_["language"])
        elif include_empty:
            instance.language = ""
        if "region" in dict_ and dict_["region"] is not None:
            instance.region = str(dict_["region"])
        elif include_empty:
            instance.region = ""
        if "returnUrl" in dict_ and dict_["returnUrl"] is not None:
            instance.return_url = str(dict_["returnUrl"])
        elif include_empty:
            instance.return_url = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, OrderCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[OrderCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[OrderCreate, List[OrderCreate], Dict[Any, OrderCreate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currencyCode": "currency_code",
            "discountedPrice": "discounted_price",
            "itemId": "item_id",
            "price": "price",
            "quantity": "quantity",
            "ext": "ext",
            "language": "language",
            "region": "region",
            "returnUrl": "return_url",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currencyCode": True,
            "discountedPrice": True,
            "itemId": True,
            "price": True,
            "quantity": True,
            "ext": False,
            "language": False,
            "region": False,
            "returnUrl": False,
        }

    @staticmethod
    def get_pattern_map() -> Dict[str, re.Pattern]:
        return {
            "language": re.compile(
                r"^[A-Za-z]{2,4}([_-][A-Za-z]{4})?([_-]([A-Za-z]{2}|[0-9]{3}))?$"
            ),
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Seasonpass Service (1.16.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.image import Image
from ..models.localization import Localization
from ..models.pass_info import PassInfo
from ..models.reward_info import RewardInfo
from ..models.tier import Tier


class FullSeasonInfo(Model):
    """A DTO object for season full content. (FullSeasonInfo)

    Properties:
        id_: (id) REQUIRED str

        localizations: (localizations) REQUIRED Dict[str, Localization]

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        passes: (passes) REQUIRED List[PassInfo]

        rewards: (rewards) REQUIRED Dict[str, RewardInfo]

        tiers: (tiers) REQUIRED List[Tier]

        images: (images) OPTIONAL List[Image]
    """

    # region fields

    id_: str  # REQUIRED
    localizations: Dict[str, Localization]  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    passes: List[PassInfo]  # REQUIRED
    rewards: Dict[str, RewardInfo]  # REQUIRED
    tiers: List[Tier]  # REQUIRED
    images: List[Image]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_id(self, value: str) -> FullSeasonInfo:
        self.id_ = value
        return self

    def with_localizations(self, value: Dict[str, Localization]) -> FullSeasonInfo:
        self.localizations = value
        return self

    def with_name(self, value: str) -> FullSeasonInfo:
        self.name = value
        return self

    def with_namespace(self, value: str) -> FullSeasonInfo:
        self.namespace = value
        return self

    def with_passes(self, value: List[PassInfo]) -> FullSeasonInfo:
        self.passes = value
        return self

    def with_rewards(self, value: Dict[str, RewardInfo]) -> FullSeasonInfo:
        self.rewards = value
        return self

    def with_tiers(self, value: List[Tier]) -> FullSeasonInfo:
        self.tiers = value
        return self

    def with_images(self, value: List[Image]) -> FullSeasonInfo:
        self.images = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "localizations"):
            result["localizations"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.localizations.items()
            }
        elif include_empty:
            result["localizations"] = {}
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "passes"):
            result["passes"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.passes
            ]
        elif include_empty:
            result["passes"] = []
        if hasattr(self, "rewards"):
            result["rewards"] = {
                str(k0): v0.to_dict(include_empty=include_empty)
                for k0, v0 in self.rewards.items()
            }
        elif include_empty:
            result["rewards"] = {}
        if hasattr(self, "tiers"):
            result["tiers"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.tiers
            ]
        elif include_empty:
            result["tiers"] = []
        if hasattr(self, "images"):
            result["images"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.images
            ]
        elif include_empty:
            result["images"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        id_: str,
        localizations: Dict[str, Localization],
        name: str,
        namespace: str,
        passes: List[PassInfo],
        rewards: Dict[str, RewardInfo],
        tiers: List[Tier],
        images: Optional[List[Image]] = None,
    ) -> FullSeasonInfo:
        instance = cls()
        instance.id_ = id_
        instance.localizations = localizations
        instance.name = name
        instance.namespace = namespace
        instance.passes = passes
        instance.rewards = rewards
        instance.tiers = tiers
        if images is not None:
            instance.images = images
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FullSeasonInfo:
        instance = cls()
        if not dict_:
            return instance
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "localizations" in dict_ and dict_["localizations"] is not None:
            instance.localizations = {
                str(k0): Localization.create_from_dict(v0, include_empty=include_empty)
                for k0, v0 in dict_["localizations"].items()
            }
        elif include_empty:
            instance.localizations = {}
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "passes" in dict_ and dict_["passes"] is not None:
            instance.passes = [
                PassInfo.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["passes"]
            ]
        elif include_empty:
            instance.passes = []
        if "rewards" in dict_ and dict_["rewards"] is not None:
            instance.rewards = {
                str(k0): RewardInfo.create_from_dict(v0, include_empty=include_empty)
                for k0, v0 in dict_["rewards"].items()
            }
        elif include_empty:
            instance.rewards = {}
        if "tiers" in dict_ and dict_["tiers"] is not None:
            instance.tiers = [
                Tier.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["tiers"]
            ]
        elif include_empty:
            instance.tiers = []
        if "images" in dict_ and dict_["images"] is not None:
            instance.images = [
                Image.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["images"]
            ]
        elif include_empty:
            instance.images = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FullSeasonInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FullSeasonInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[FullSeasonInfo, List[FullSeasonInfo], Dict[Any, FullSeasonInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "id": "id_",
            "localizations": "localizations",
            "name": "name",
            "namespace": "namespace",
            "passes": "passes",
            "rewards": "rewards",
            "tiers": "tiers",
            "images": "images",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "id": True,
            "localizations": True,
            "name": True,
            "namespace": True,
            "passes": True,
            "rewards": True,
            "tiers": True,
            "images": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Platform Service (4.17.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model
from ....core import StrEnum

from ..models.redeemable_item import RedeemableItem


class RedeemTypeEnum(StrEnum):
    ITEM = "ITEM"


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"


class TypeEnum(StrEnum):
    REDEMPTION = "REDEMPTION"


class CampaignInfo(Model):
    """Campaign info (CampaignInfo)

    Properties:
        booth_name: (boothName) REQUIRED str

        created_at: (createdAt) REQUIRED str

        id_: (id) REQUIRED str

        max_redeem_count_per_campaign_per_user: (maxRedeemCountPerCampaignPerUser) REQUIRED int

        max_redeem_count_per_code: (maxRedeemCountPerCode) REQUIRED int

        max_redeem_count_per_code_per_user: (maxRedeemCountPerCodePerUser) REQUIRED int

        max_sale_count: (maxSaleCount) REQUIRED int

        name: (name) REQUIRED str

        namespace: (namespace) REQUIRED str

        redeem_type: (redeemType) REQUIRED Union[str, RedeemTypeEnum]

        status: (status) REQUIRED Union[str, StatusEnum]

        type_: (type) REQUIRED Union[str, TypeEnum]

        updated_at: (updatedAt) REQUIRED str

        description: (description) OPTIONAL str

        items: (items) OPTIONAL List[RedeemableItem]

        redeem_end: (redeemEnd) OPTIONAL str

        redeem_start: (redeemStart) OPTIONAL str

        tags: (tags) OPTIONAL List[str]
    """

    # region fields

    booth_name: str  # REQUIRED
    created_at: str  # REQUIRED
    id_: str  # REQUIRED
    max_redeem_count_per_campaign_per_user: int  # REQUIRED
    max_redeem_count_per_code: int  # REQUIRED
    max_redeem_count_per_code_per_user: int  # REQUIRED
    max_sale_count: int  # REQUIRED
    name: str  # REQUIRED
    namespace: str  # REQUIRED
    redeem_type: Union[str, RedeemTypeEnum]  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    type_: Union[str, TypeEnum]  # REQUIRED
    updated_at: str  # REQUIRED
    description: str  # OPTIONAL
    items: List[RedeemableItem]  # OPTIONAL
    redeem_end: str  # OPTIONAL
    redeem_start: str  # OPTIONAL
    tags: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_booth_name(self, value: str) -> CampaignInfo:
        self.booth_name = value
        return self

    def with_created_at(self, value: str) -> CampaignInfo:
        self.created_at = value
        return self

    def with_id(self, value: str) -> CampaignInfo:
        self.id_ = value
        return self

    def with_max_redeem_count_per_campaign_per_user(self, value: int) -> CampaignInfo:
        self.max_redeem_count_per_campaign_per_user = value
        return self

    def with_max_redeem_count_per_code(self, value: int) -> CampaignInfo:
        self.max_redeem_count_per_code = value
        return self

    def with_max_redeem_count_per_code_per_user(self, value: int) -> CampaignInfo:
        self.max_redeem_count_per_code_per_user = value
        return self

    def with_max_sale_count(self, value: int) -> CampaignInfo:
        self.max_sale_count = value
        return self

    def with_name(self, value: str) -> CampaignInfo:
        self.name = value
        return self

    def with_namespace(self, value: str) -> CampaignInfo:
        self.namespace = value
        return self

    def with_redeem_type(self, value: Union[str, RedeemTypeEnum]) -> CampaignInfo:
        self.redeem_type = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> CampaignInfo:
        self.status = value
        return self

    def with_type(self, value: Union[str, TypeEnum]) -> CampaignInfo:
        self.type_ = value
        return self

    def with_updated_at(self, value: str) -> CampaignInfo:
        self.updated_at = value
        return self

    def with_description(self, value: str) -> CampaignInfo:
        self.description = value
        return self

    def with_items(self, value: List[RedeemableItem]) -> CampaignInfo:
        self.items = value
        return self

    def with_redeem_end(self, value: str) -> CampaignInfo:
        self.redeem_end = value
        return self

    def with_redeem_start(self, value: str) -> CampaignInfo:
        self.redeem_start = value
        return self

    def with_tags(self, value: List[str]) -> CampaignInfo:
        self.tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "booth_name"):
            result["boothName"] = str(self.booth_name)
        elif include_empty:
            result["boothName"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "max_redeem_count_per_campaign_per_user"):
            result["maxRedeemCountPerCampaignPerUser"] = int(
                self.max_redeem_count_per_campaign_per_user
            )
        elif include_empty:
            result["maxRedeemCountPerCampaignPerUser"] = 0
        if hasattr(self, "max_redeem_count_per_code"):
            result["maxRedeemCountPerCode"] = int(self.max_redeem_count_per_code)
        elif include_empty:
            result["maxRedeemCountPerCode"] = 0
        if hasattr(self, "max_redeem_count_per_code_per_user"):
            result["maxRedeemCountPerCodePerUser"] = int(
                self.max_redeem_count_per_code_per_user
            )
        elif include_empty:
            result["maxRedeemCountPerCodePerUser"] = 0
        if hasattr(self, "max_sale_count"):
            result["maxSaleCount"] = int(self.max_sale_count)
        elif include_empty:
            result["maxSaleCount"] = 0
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "redeem_type"):
            result["redeemType"] = str(self.redeem_type)
        elif include_empty:
            result["redeemType"] = Union[str, RedeemTypeEnum]()
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = Union[str, TypeEnum]()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "items"):
            result["items"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.items
            ]
        elif include_empty:
            result["items"] = []
        if hasattr(self, "redeem_end"):
            result["redeemEnd"] = str(self.redeem_end)
        elif include_empty:
            result["redeemEnd"] = ""
        if hasattr(self, "redeem_start"):
            result["redeemStart"] = str(self.redeem_start)
        elif include_empty:
            result["redeemStart"] = ""
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        booth_name: str,
        created_at: str,
        id_: str,
        max_redeem_count_per_campaign_per_user: int,
        max_redeem_count_per_code: int,
        max_redeem_count_per_code_per_user: int,
        max_sale_count: int,
        name: str,
        namespace: str,
        redeem_type: Union[str, RedeemTypeEnum],
        status: Union[str, StatusEnum],
        type_: Union[str, TypeEnum],
        updated_at: str,
        description: Optional[str] = None,
        items: Optional[List[RedeemableItem]] = None,
        redeem_end: Optional[str] = None,
        redeem_start: Optional[str] = None,
        tags: Optional[List[str]] = None,
    ) -> CampaignInfo:
        instance = cls()
        instance.booth_name = booth_name
        instance.created_at = created_at
        instance.id_ = id_
        instance.max_redeem_count_per_campaign_per_user = (
            max_redeem_count_per_campaign_per_user
        )
        instance.max_redeem_count_per_code = max_redeem_count_per_code
        instance.max_redeem_count_per_code_per_user = max_redeem_count_per_code_per_user
        instance.max_sale_count = max_sale_count
        instance.name = name
        instance.namespace = namespace
        instance.redeem_type = redeem_type
        instance.status = status
        instance.type_ = type_
        instance.updated_at = updated_at
        if description is not None:
            instance.description = description
        if items is not None:
            instance.items = items
        if redeem_end is not None:
            instance.redeem_end = redeem_end
        if redeem_start is not None:
            instance.redeem_start = redeem_start
        if tags is not None:
            instance.tags = tags
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> CampaignInfo:
        instance = cls()
        if not dict_:
            return instance
        if "boothName" in dict_ and dict_["boothName"] is not None:
            instance.booth_name = str(dict_["boothName"])
        elif include_empty:
            instance.booth_name = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if (
            "maxRedeemCountPerCampaignPerUser" in dict_
            and dict_["maxRedeemCountPerCampaignPerUser"] is not None
        ):
            instance.max_redeem_count_per_campaign_per_user = int(
                dict_["maxRedeemCountPerCampaignPerUser"]
            )
        elif include_empty:
            instance.max_redeem_count_per_campaign_per_user = 0
        if (
            "maxRedeemCountPerCode" in dict_
            and dict_["maxRedeemCountPerCode"] is not None
        ):
            instance.max_redeem_count_per_code = int(dict_["maxRedeemCountPerCode"])
        elif include_empty:
            instance.max_redeem_count_per_code = 0
        if (
            "maxRedeemCountPerCodePerUser" in dict_
            and dict_["maxRedeemCountPerCodePerUser"] is not None
        ):
            instance.max_redeem_count_per_code_per_user = int(
                dict_["maxRedeemCountPerCodePerUser"]
            )
        elif include_empty:
            instance.max_redeem_count_per_code_per_user = 0
        if "maxSaleCount" in dict_ and dict_["maxSaleCount"] is not None:
            instance.max_sale_count = int(dict_["maxSaleCount"])
        elif include_empty:
            instance.max_sale_count = 0
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "redeemType" in dict_ and dict_["redeemType"] is not None:
            instance.redeem_type = str(dict_["redeemType"])
        elif include_empty:
            instance.redeem_type = Union[str, RedeemTypeEnum]()
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = Union[str, TypeEnum]()
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "items" in dict_ and dict_["items"] is not None:
            instance.items = [
                RedeemableItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["items"]
            ]
        elif include_empty:
            instance.items = []
        if "redeemEnd" in dict_ and dict_["redeemEnd"] is not None:
            instance.redeem_end = str(dict_["redeemEnd"])
        elif include_empty:
            instance.redeem_end = ""
        if "redeemStart" in dict_ and dict_["redeemStart"] is not None:
            instance.redeem_start = str(dict_["redeemStart"])
        elif include_empty:
            instance.redeem_start = ""
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, CampaignInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[CampaignInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[CampaignInfo, List[CampaignInfo], Dict[Any, CampaignInfo]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "boothName": "booth_name",
            "createdAt": "created_at",
            "id": "id_",
            "maxRedeemCountPerCampaignPerUser": "max_redeem_count_per_campaign_per_user",
            "maxRedeemCountPerCode": "max_redeem_count_per_code",
            "maxRedeemCountPerCodePerUser": "max_redeem_count_per_code_per_user",
            "maxSaleCount": "max_sale_count",
            "name": "name",
            "namespace": "namespace",
            "redeemType": "redeem_type",
            "status": "status",
            "type": "type_",
            "updatedAt": "updated_at",
            "description": "description",
            "items": "items",
            "redeemEnd": "redeem_end",
            "redeemStart": "redeem_start",
            "tags": "tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "boothName": True,
            "createdAt": True,
            "id": True,
            "maxRedeemCountPerCampaignPerUser": True,
            "maxRedeemCountPerCode": True,
            "maxRedeemCountPerCodePerUser": True,
            "maxSaleCount": True,
            "name": True,
            "namespace": True,
            "redeemType": True,
            "status": True,
            "type": True,
            "updatedAt": True,
            "description": False,
            "items": False,
            "redeemEnd": False,
            "redeemStart": False,
            "tags": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "redeemType": ["ITEM"],
            "status": ["ACTIVE", "INACTIVE"],
            "type": ["REDEMPTION"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Leaderboard Service."""

__version__ = "2.15.9"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .models_archive_leaderboard_req import ModelsArchiveLeaderboardReq
from .models_archive_leaderboard_signed_url_response import (
    ModelsArchiveLeaderboardSignedURLResponse,
)
from .models_daily_config import ModelsDailyConfig
from .models_delete_bulk_leaderboard_failed_resp import (
    ModelsDeleteBulkLeaderboardFailedResp,
)
from .models_delete_bulk_leaderboards_req import ModelsDeleteBulkLeaderboardsReq
from .models_delete_bulk_leaderboards_resp import ModelsDeleteBulkLeaderboardsResp
from .models_get_all_leaderboard_configs_public_resp import (
    ModelsGetAllLeaderboardConfigsPublicResp,
)
from .models_get_all_leaderboard_configs_resp import ModelsGetAllLeaderboardConfigsResp
from .models_get_all_user_leaderboards_resp import ModelsGetAllUserLeaderboardsResp
from .models_get_hidden_user_response import ModelsGetHiddenUserResponse
from .models_get_leaderboard_config_public_resp import (
    ModelsGetLeaderboardConfigPublicResp,
)
from .models_get_leaderboard_config_resp import ModelsGetLeaderboardConfigResp
from .models_get_leaderboard_ranking_resp import ModelsGetLeaderboardRankingResp
from .models_get_user_visibility_response import ModelsGetUserVisibilityResponse
from .models_leaderboard_config_req import ModelsLeaderboardConfigReq
from .models_monthly_config import ModelsMonthlyConfig
from .models_pagination import ModelsPagination
from .models_set_user_visibility_request import ModelsSetUserVisibilityRequest
from .models_update_leaderboard_config_req import ModelsUpdateLeaderboardConfigReq
from .models_update_user_point_admin_v1_request import (
    ModelsUpdateUserPointAdminV1Request,
)
from .models_update_user_point_admin_v1_response import (
    ModelsUpdateUserPointAdminV1Response,
)
from .models_user_leaderboard_ranking import ModelsUserLeaderboardRanking
from .models_user_point import ModelsUserPoint
from .models_user_ranking_response import ModelsUserRankingResponse
from .models_user_ranking_response_detail import ModelsUserRankingResponseDetail
from .models_weekly_config import ModelsWeeklyConfig
from .response_error_response import ResponseErrorResponse
from .v2_entry import V2Entry
from .v2_get_all_leaderboard_configs_public_resp import (
    V2GetAllLeaderboardConfigsPublicResp,
)
from .v2_get_leaderboard_config_public_resp import V2GetLeaderboardConfigPublicResp
from .v2_get_public_leaderboard_ranking_response import (
    V2GetPublicLeaderboardRankingResponse,
)
from .v2_pagination import V2Pagination



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

"""Auto-generated package that contains models used by the AccelByte Cloud Iam Service."""

__version__ = "5.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .authorization import Authorization
from .authorization import ResponseTypeEnum as AuthorizationResponseTypeEnum
from .get_jwks import GetJWKS
from .get_revocation_list import GetRevocationList
from .platform_token_request_handler import PlatformTokenRequestHandler
from .revoke_a_user import RevokeAUser
from .revoke_token import RevokeToken
from .revoke_user import RevokeUser
from .token_grant import TokenGrant
from .token_grant import GrantTypeEnum as TokenGrantGrantTypeEnum
from .verify_token import VerifyToken



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Iam Service (5.20.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_platform_user_information_v3 import (
    AccountcommonPlatformUserInformationV3,
)


class AccountcommonUserInformationV3(Model):
    """Accountcommon user information V3 (accountcommon.UserInformationV3)

    Properties:
        email_addresses: (emailAddresses) REQUIRED List[str]

        platform_users: (platformUsers) REQUIRED List[AccountcommonPlatformUserInformationV3]

        country: (country) OPTIONAL str

        display_name: (displayName) OPTIONAL str

        phone_number: (phoneNumber) OPTIONAL str

        username: (username) OPTIONAL str

        xbox_user_id: (xboxUserId) OPTIONAL str
    """

    # region fields

    email_addresses: List[str]  # REQUIRED
    platform_users: List[AccountcommonPlatformUserInformationV3]  # REQUIRED
    country: str  # OPTIONAL
    display_name: str  # OPTIONAL
    phone_number: str  # OPTIONAL
    username: str  # OPTIONAL
    xbox_user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_email_addresses(self, value: List[str]) -> AccountcommonUserInformationV3:
        self.email_addresses = value
        return self

    def with_platform_users(
        self, value: List[AccountcommonPlatformUserInformationV3]
    ) -> AccountcommonUserInformationV3:
        self.platform_users = value
        return self

    def with_country(self, value: str) -> AccountcommonUserInformationV3:
        self.country = value
        return self

    def with_display_name(self, value: str) -> AccountcommonUserInformationV3:
        self.display_name = value
        return self

    def with_phone_number(self, value: str) -> AccountcommonUserInformationV3:
        self.phone_number = value
        return self

    def with_username(self, value: str) -> AccountcommonUserInformationV3:
        self.username = value
        return self

    def with_xbox_user_id(self, value: str) -> AccountcommonUserInformationV3:
        self.xbox_user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "email_addresses"):
            result["emailAddresses"] = [str(i0) for i0 in self.email_addresses]
        elif include_empty:
            result["emailAddresses"] = []
        if hasattr(self, "platform_users"):
            result["platformUsers"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.platform_users
            ]
        elif include_empty:
            result["platformUsers"] = []
        if hasattr(self, "country"):
            result["country"] = str(self.country)
        elif include_empty:
            result["country"] = ""
        if hasattr(self, "display_name"):
            result["displayName"] = str(self.display_name)
        elif include_empty:
            result["displayName"] = ""
        if hasattr(self, "phone_number"):
            result["phoneNumber"] = str(self.phone_number)
        elif include_empty:
            result["phoneNumber"] = ""
        if hasattr(self, "username"):
            result["username"] = str(self.username)
        elif include_empty:
            result["username"] = ""
        if hasattr(self, "xbox_user_id"):
            result["xboxUserId"] = str(self.xbox_user_id)
        elif include_empty:
            result["xboxUserId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        email_addresses: List[str],
        platform_users: List[AccountcommonPlatformUserInformationV3],
        country: Optional[str] = None,
        display_name: Optional[str] = None,
        phone_number: Optional[str] = None,
        username: Optional[str] = None,
        xbox_user_id: Optional[str] = None,
    ) -> AccountcommonUserInformationV3:
        instance = cls()
        instance.email_addresses = email_addresses
        instance.platform_users = platform_users
        if country is not None:
            instance.country = country
        if display_name is not None:
            instance.display_name = display_name
        if phone_number is not None:
            instance.phone_number = phone_number
        if username is not None:
            instance.username = username
        if xbox_user_id is not None:
            instance.xbox_user_id = xbox_user_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AccountcommonUserInformationV3:
        instance = cls()
        if not dict_:
            return instance
        if "emailAddresses" in dict_ and dict_["emailAddresses"] is not None:
            instance.email_addresses = [str(i0) for i0 in dict_["emailAddresses"]]
        elif include_empty:
            instance.email_addresses = []
        if "platformUsers" in dict_ and dict_["platformUsers"] is not None:
            instance.platform_users = [
                AccountcommonPlatformUserInformationV3.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["platformUsers"]
            ]
        elif include_empty:
            instance.platform_users = []
        if "country" in dict_ and dict_["country"] is not None:
            instance.country = str(dict_["country"])
        elif include_empty:
            instance.country = ""
        if "displayName" in dict_ and dict_["displayName"] is not None:
            instance.display_name = str(dict_["displayName"])
        elif include_empty:
            instance.display_name = ""
        if "phoneNumber" in dict_ and dict_["phoneNumber"] is not None:
            instance.phone_number = str(dict_["phoneNumber"])
        elif include_empty:
            instance.phone_number = ""
        if "username" in dict_ and dict_["username"] is not None:
            instance.username = str(dict_["username"])
        elif include_empty:
            instance.username = ""
        if "xboxUserId" in dict_ and dict_["xboxUserId"] is not None:
            instance.xbox_user_id = str(dict_["xboxUserId"])
        elif include_empty:
            instance.xbox_user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AccountcommonUserInformationV3]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AccountcommonUserInformationV3]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        AccountcommonUserInformationV3,
        List[AccountcommonUserInformationV3],
        Dict[Any, AccountcommonUserInformationV3],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "emailAddresses": "email_addresses",
            "platformUsers": "platform_users",
            "country": "country",
            "displayName": "display_name",
            "phoneNumber": "phone_number",
            "username": "username",
            "xboxUserId": "xbox_user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "emailAddresses": True,
            "platformUsers": True,
            "country": False,
            "displayName": False,
            "phoneNumber": False,
            "username": False,
            "xboxUserId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsConfigExport
from ..models import ModelsConfigList
from ..models import ModelsConfigReq
from ..models import ModelsImportConfigResponse
from ..models import ResponseError
from ..models import RestapiErrorResponseBody

from ..operations.config import AdminExportConfigV1
from ..operations.config import AdminGetAllConfigV1
from ..operations.config import AdminGetConfigV1
from ..operations.config import AdminImportConfigV1
from ..operations.config import AdminUpdateConfigV1


@same_doc_as(AdminExportConfigV1)
def admin_export_config_v1(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminExportConfigV1.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminExportConfigV1)
async def admin_export_config_v1_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminExportConfigV1.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetAllConfigV1)
def admin_get_all_config_v1(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = AdminGetAllConfigV1.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetAllConfigV1)
async def admin_get_all_config_v1_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    request = AdminGetAllConfigV1.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminGetConfigV1)
def admin_get_config_v1(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetConfigV1.create(
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminGetConfigV1)
async def admin_get_config_v1_async(
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminGetConfigV1.create(
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminImportConfigV1)
def admin_import_config_v1(
    file: Optional[Any] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminImportConfigV1.create(
        file=file,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminImportConfigV1)
async def admin_import_config_v1_async(
    file: Optional[Any] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminImportConfigV1.create(
        file=file,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminUpdateConfigV1)
def admin_update_config_v1(
    body: ModelsConfigReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateConfigV1.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminUpdateConfigV1)
async def admin_update_config_v1_async(
    body: ModelsConfigReq,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = AdminUpdateConfigV1.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Matchmaking Service (2.17.2)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.models_combination_alliances import ModelsCombinationAlliances


class ModelsCombination(Model):
    """Models combination (models.Combination)

    Properties:
        alliances: (alliances) REQUIRED List[ModelsCombinationAlliances]

        has_combination: (has_combination) REQUIRED bool

        role_flexing_enable: (role_flexing_enable) REQUIRED bool

        role_flexing_player: (role_flexing_player) REQUIRED int

        role_flexing_second: (role_flexing_second) REQUIRED int
    """

    # region fields

    alliances: List[ModelsCombinationAlliances]  # REQUIRED
    has_combination: bool  # REQUIRED
    role_flexing_enable: bool  # REQUIRED
    role_flexing_player: int  # REQUIRED
    role_flexing_second: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_alliances(
        self, value: List[ModelsCombinationAlliances]
    ) -> ModelsCombination:
        self.alliances = value
        return self

    def with_has_combination(self, value: bool) -> ModelsCombination:
        self.has_combination = value
        return self

    def with_role_flexing_enable(self, value: bool) -> ModelsCombination:
        self.role_flexing_enable = value
        return self

    def with_role_flexing_player(self, value: int) -> ModelsCombination:
        self.role_flexing_player = value
        return self

    def with_role_flexing_second(self, value: int) -> ModelsCombination:
        self.role_flexing_second = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "alliances"):
            result["alliances"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.alliances
            ]
        elif include_empty:
            result["alliances"] = []
        if hasattr(self, "has_combination"):
            result["has_combination"] = bool(self.has_combination)
        elif include_empty:
            result["has_combination"] = False
        if hasattr(self, "role_flexing_enable"):
            result["role_flexing_enable"] = bool(self.role_flexing_enable)
        elif include_empty:
            result["role_flexing_enable"] = False
        if hasattr(self, "role_flexing_player"):
            result["role_flexing_player"] = int(self.role_flexing_player)
        elif include_empty:
            result["role_flexing_player"] = 0
        if hasattr(self, "role_flexing_second"):
            result["role_flexing_second"] = int(self.role_flexing_second)
        elif include_empty:
            result["role_flexing_second"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        alliances: List[ModelsCombinationAlliances],
        has_combination: bool,
        role_flexing_enable: bool,
        role_flexing_player: int,
        role_flexing_second: int,
    ) -> ModelsCombination:
        instance = cls()
        instance.alliances = alliances
        instance.has_combination = has_combination
        instance.role_flexing_enable = role_flexing_enable
        instance.role_flexing_player = role_flexing_player
        instance.role_flexing_second = role_flexing_second
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelsCombination:
        instance = cls()
        if not dict_:
            return instance
        if "alliances" in dict_ and dict_["alliances"] is not None:
            instance.alliances = [
                ModelsCombinationAlliances.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["alliances"]
            ]
        elif include_empty:
            instance.alliances = []
        if "has_combination" in dict_ and dict_["has_combination"] is not None:
            instance.has_combination = bool(dict_["has_combination"])
        elif include_empty:
            instance.has_combination = False
        if "role_flexing_enable" in dict_ and dict_["role_flexing_enable"] is not None:
            instance.role_flexing_enable = bool(dict_["role_flexing_enable"])
        elif include_empty:
            instance.role_flexing_enable = False
        if "role_flexing_player" in dict_ and dict_["role_flexing_player"] is not None:
            instance.role_flexing_player = int(dict_["role_flexing_player"])
        elif include_empty:
            instance.role_flexing_player = 0
        if "role_flexing_second" in dict_ and dict_["role_flexing_second"] is not None:
            instance.role_flexing_second = int(dict_["role_flexing_second"])
        elif include_empty:
            instance.role_flexing_second = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelsCombination]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelsCombination]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelsCombination, List[ModelsCombination], Dict[Any, ModelsCombination]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "alliances": "alliances",
            "has_combination": "has_combination",
            "role_flexing_enable": "role_flexing_enable",
            "role_flexing_player": "role_flexing_player",
            "role_flexing_second": "role_flexing_second",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "alliances": True,
            "has_combination": True,
            "role_flexing_enable": True,
            "role_flexing_player": True,
            "role_flexing_second": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# AccelByte Cloud Social Service (1.30.1)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class GameProfileRequest(Model):
    """Game profile request (GameProfileRequest)

    Properties:
        achievements: (achievements) OPTIONAL List[str]

        attributes: (attributes) OPTIONAL Dict[str, str]

        avatar_url: (avatarUrl) OPTIONAL str

        inventories: (inventories) OPTIONAL List[str]

        label: (label) OPTIONAL str

        profile_name: (profileName) OPTIONAL str

        statistics: (statistics) OPTIONAL List[str]

        tags: (tags) OPTIONAL List[str]
    """

    # region fields

    achievements: List[str]  # OPTIONAL
    attributes: Dict[str, str]  # OPTIONAL
    avatar_url: str  # OPTIONAL
    inventories: List[str]  # OPTIONAL
    label: str  # OPTIONAL
    profile_name: str  # OPTIONAL
    statistics: List[str]  # OPTIONAL
    tags: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_achievements(self, value: List[str]) -> GameProfileRequest:
        self.achievements = value
        return self

    def with_attributes(self, value: Dict[str, str]) -> GameProfileRequest:
        self.attributes = value
        return self

    def with_avatar_url(self, value: str) -> GameProfileRequest:
        self.avatar_url = value
        return self

    def with_inventories(self, value: List[str]) -> GameProfileRequest:
        self.inventories = value
        return self

    def with_label(self, value: str) -> GameProfileRequest:
        self.label = value
        return self

    def with_profile_name(self, value: str) -> GameProfileRequest:
        self.profile_name = value
        return self

    def with_statistics(self, value: List[str]) -> GameProfileRequest:
        self.statistics = value
        return self

    def with_tags(self, value: List[str]) -> GameProfileRequest:
        self.tags = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "achievements"):
            result["achievements"] = [str(i0) for i0 in self.achievements]
        elif include_empty:
            result["achievements"] = []
        if hasattr(self, "attributes"):
            result["attributes"] = {
                str(k0): str(v0) for k0, v0 in self.attributes.items()
            }
        elif include_empty:
            result["attributes"] = {}
        if hasattr(self, "avatar_url"):
            result["avatarUrl"] = str(self.avatar_url)
        elif include_empty:
            result["avatarUrl"] = ""
        if hasattr(self, "inventories"):
            result["inventories"] = [str(i0) for i0 in self.inventories]
        elif include_empty:
            result["inventories"] = []
        if hasattr(self, "label"):
            result["label"] = str(self.label)
        elif include_empty:
            result["label"] = ""
        if hasattr(self, "profile_name"):
            result["profileName"] = str(self.profile_name)
        elif include_empty:
            result["profileName"] = ""
        if hasattr(self, "statistics"):
            result["statistics"] = [str(i0) for i0 in self.statistics]
        elif include_empty:
            result["statistics"] = []
        if hasattr(self, "tags"):
            result["tags"] = [str(i0) for i0 in self.tags]
        elif include_empty:
            result["tags"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        achievements: Optional[List[str]] = None,
        attributes: Optional[Dict[str, str]] = None,
        avatar_url: Optional[str] = None,
        inventories: Optional[List[str]] = None,
        label: Optional[str] = None,
        profile_name: Optional[str] = None,
        statistics: Optional[List[str]] = None,
        tags: Optional[List[str]] = None,
    ) -> GameProfileRequest:
        instance = cls()
        if achievements is not None:
            instance.achievements = achievements
        if attributes is not None:
            instance.attributes = attributes
        if avatar_url is not None:
            instance.avatar_url = avatar_url
        if inventories is not None:
            instance.inventories = inventories
        if label is not None:
            instance.label = label
        if profile_name is not None:
            instance.profile_name = profile_name
        if statistics is not None:
            instance.statistics = statistics
        if tags is not None:
            instance.tags = tags
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> GameProfileRequest:
        instance = cls()
        if not dict_:
            return instance
        if "achievements" in dict_ and dict_["achievements"] is not None:
            instance.achievements = [str(i0) for i0 in dict_["achievements"]]
        elif include_empty:
            instance.achievements = []
        if "attributes" in dict_ and dict_["attributes"] is not None:
            instance.attributes = {
                str(k0): str(v0) for k0, v0 in dict_["attributes"].items()
            }
        elif include_empty:
            instance.attributes = {}
        if "avatarUrl" in dict_ and dict_["avatarUrl"] is not None:
            instance.avatar_url = str(dict_["avatarUrl"])
        elif include_empty:
            instance.avatar_url = ""
        if "inventories" in dict_ and dict_["inventories"] is not None:
            instance.inventories = [str(i0) for i0 in dict_["inventories"]]
        elif include_empty:
            instance.inventories = []
        if "label" in dict_ and dict_["label"] is not None:
            instance.label = str(dict_["label"])
        elif include_empty:
            instance.label = ""
        if "profileName" in dict_ and dict_["profileName"] is not None:
            instance.profile_name = str(dict_["profileName"])
        elif include_empty:
            instance.profile_name = ""
        if "statistics" in dict_ and dict_["statistics"] is not None:
            instance.statistics = [str(i0) for i0 in dict_["statistics"]]
        elif include_empty:
            instance.statistics = []
        if "tags" in dict_ and dict_["tags"] is not None:
            instance.tags = [str(i0) for i0 in dict_["tags"]]
        elif include_empty:
            instance.tags = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, GameProfileRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[GameProfileRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        GameProfileRequest, List[GameProfileRequest], Dict[Any, GameProfileRequest]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "achievements": "achievements",
            "attributes": "attributes",
            "avatarUrl": "avatar_url",
            "inventories": "inventories",
            "label": "label",
            "profileName": "profile_name",
            "statistics": "statistics",
            "tags": "tags",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "achievements": False,
            "attributes": False,
            "avatarUrl": False,
            "inventories": False,
            "label": False,
            "profileName": False,
            "statistics": False,
            "tags": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: accelbyte_cloud_py_codegen

# Analytics Game Telemetry (1.11.0)

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class TelemetryBody(Model):
    """Telemetry body (TelemetryBody)

    Properties:
        event_name: (EventName) REQUIRED str

        event_namespace: (EventNamespace) REQUIRED str

        payload: (Payload) REQUIRED Dict[str, Any]

        event_id: (EventId) OPTIONAL str

        event_timestamp: (EventTimestamp) OPTIONAL str
    """

    # region fields

    event_name: str  # REQUIRED
    event_namespace: str  # REQUIRED
    payload: Dict[str, Any]  # REQUIRED
    event_id: str  # OPTIONAL
    event_timestamp: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_event_name(self, value: str) -> TelemetryBody:
        self.event_name = value
        return self

    def with_event_namespace(self, value: str) -> TelemetryBody:
        self.event_namespace = value
        return self

    def with_payload(self, value: Dict[str, Any]) -> TelemetryBody:
        self.payload = value
        return self

    def with_event_id(self, value: str) -> TelemetryBody:
        self.event_id = value
        return self

    def with_event_timestamp(self, value: str) -> TelemetryBody:
        self.event_timestamp = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "event_name"):
            result["EventName"] = str(self.event_name)
        elif include_empty:
            result["EventName"] = ""
        if hasattr(self, "event_namespace"):
            result["EventNamespace"] = str(self.event_namespace)
        elif include_empty:
            result["EventNamespace"] = ""
        if hasattr(self, "payload"):
            result["Payload"] = {str(k0): v0 for k0, v0 in self.payload.items()}
        elif include_empty:
            result["Payload"] = {}
        if hasattr(self, "event_id"):
            result["EventId"] = str(self.event_id)
        elif include_empty:
            result["EventId"] = ""
        if hasattr(self, "event_timestamp"):
            result["EventTimestamp"] = str(self.event_timestamp)
        elif include_empty:
            result["EventTimestamp"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        event_name: str,
        event_namespace: str,
        payload: Dict[str, Any],
        event_id: Optional[str] = None,
        event_timestamp: Optional[str] = None,
    ) -> TelemetryBody:
        instance = cls()
        instance.event_name = event_name
        instance.event_namespace = event_namespace
        instance.payload = payload
        if event_id is not None:
            instance.event_id = event_id
        if event_timestamp is not None:
            instance.event_timestamp = event_timestamp
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> TelemetryBody:
        instance = cls()
        if not dict_:
            return instance
        if "EventName" in dict_ and dict_["EventName"] is not None:
            instance.event_name = str(dict_["EventName"])
        elif include_empty:
            instance.event_name = ""
        if "EventNamespace" in dict_ and dict_["EventNamespace"] is not None:
            instance.event_namespace = str(dict_["EventNamespace"])
        elif include_empty:
            instance.event_namespace = ""
        if "Payload" in dict_ and dict_["Payload"] is not None:
            instance.payload = {str(k0): v0 for k0, v0 in dict_["Payload"].items()}
        elif include_empty:
            instance.payload = {}
        if "EventId" in dict_ and dict_["EventId"] is not None:
            instance.event_id = str(dict_["EventId"])
        elif include_empty:
            instance.event_id = ""
        if "EventTimestamp" in dict_ and dict_["EventTimestamp"] is not None:
            instance.event_timestamp = str(dict_["EventTimestamp"])
        elif include_empty:
            instance.event_timestamp = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, TelemetryBody]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[TelemetryBody]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[TelemetryBody, List[TelemetryBody], Dict[Any, TelemetryBody]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "EventName": "event_name",
            "EventNamespace": "event_namespace",
            "Payload": "payload",
            "EventId": "event_id",
            "EventTimestamp": "event_timestamp",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "EventName": True,
            "EventNamespace": True,
            "Payload": True,
            "EventId": False,
            "EventTimestamp": False,
        }

    # endregion static methods
